﻿using RobotSimulation.RobotSimulation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RobotSimulation.Genetic
{
    public class GeneticsLab
    {
        List<Individual> archive; // stores the best of each generation
        List<Individual> generation;
        List<Individual> newGeneration;

        const int mutationSize = 100;
        const int migrationSize = 50;
        const int crossOverSize = 50;
        int generationSize = mutationSize + migrationSize + crossOverSize;

        GeneticsLab()
        {
            this.archive = new List<Individual>();
            this.generation = new List<Individual>();
            this.newGeneration = new List<Individual>();
            for(int i = 0; i < generationSize; i++)
            {
                this.generation.Add(new Individual());
            }
        }

        public double simulateIndividual(Individual i,SimulationRoom sr)
        {
            i.fitness = 0;
            while (sr.sense_line() > 0)
            {
                byte s = sr.sense_line();
                int[] sensors =new int[4];
                sensors[0] =  s & 1;
                sensors[1] =  s & 2;
                sensors[2] =  s & 4;
                sensors[3] =  s & 8;
                double[] neuralOutput = i.neuralNet.calculateNeuralNetwork(sensors);
                sr.rc.DecideMove(neuralOutput[0], neuralOutput[1]);
                i.fitness += 1;
                if(i.fitness > 1000)
                {
                    break;
                }
            }
            return i.fitness;
        }

        public void SimulateGeneration()
        {
            SimulationRoom sr = new SimulationRoom();
            foreach (Individual i in generation)
            {
                simulateIndividual(i, sr);
            }
        }

        public void makeNewGeneration()
        {
            this.generation.Sort(new Comparison<Individual>((x, y) => (x.fitness.CompareTo(y.fitness))));
            this.newGeneration.Clear();

            this.archive.Add(this.generation[generationSize-1]);
            //migracia
            for(int i = 0; i < migrationSize; i++)
            {
                this.newGeneration.Add(new Individual());
            }
            //mutacia
            for(int i= 0; i < mutationSize; i++)
            {
                Individual ind = new Individual(this.generation[generationSize - mutationSize + i].mutateNeural());
                this.newGeneration.Add(ind);
            }
            //crossover
            for(int i=0; i < crossOverSize *2; i= i+2)
            {
                this.newGeneration.Add(crossOver(generation[generationSize-crossOverSize*2+i],generation[generationSize - crossOverSize * 2 + i+1]));
            }
        }

        public Individual crossOver(Individual i1, Individual i2)
        {
            Individual ind = new Individual();
            ind.fitness = 0;
            for(int i = 0; i < i1.neuralNet.inputNeurons; i++)
            {
                for(int j =0; j < i1.neuralNet.hiddenNeurons; j++)
                {
                    ind.neuralNet.weights[j * ind.neuralNet.inputNeurons + i] = (i1.neuralNet.weights[j * ind.neuralNet.inputNeurons + i] + i2.neuralNet.weights[j * ind.neuralNet.inputNeurons + i] )/ 2;
                    for(int k =0; k < i1.neuralNet.outputNeurons; k++)
                    {
                        ind.neuralNet.outputWeights[k * i1.neuralNet.outputNeurons + j] = (i1.neuralNet.outputWeights[k * i1.neuralNet.outputNeurons + j] + i2.neuralNet.outputWeights[k * i1.neuralNet.outputNeurons + j]) / 2;
                    }
                }
            }

            return ind;
        }

        public void geneticAlgorithm(int generations)
        {
            for(int i =0; i < generations; i++)
            {
                this.SimulateGeneration();
                this.makeNewGeneration();
                this.generation.Clear();
                this.generation.AddRange(newGeneration);
            }
            this.archive.Sort(new Comparison<Individual>((x, y) => (x.fitness.CompareTo(y.fitness))));
            Console.Write(this.archive[archive.Count-1].repr());
        }
    }
}
