﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RobotSimulation.Neural_Network
{
    public class NeuralNetwork
    {
        public int inputNeurons = 4;
        public int hiddenNeurons = 3;
        public int outputNeurons = 2;

        public double[] weights;
        public double[] outputWeights;

        Random r = new Random();

        public NeuralNetwork()
        {
            weights = new double[inputNeurons * hiddenNeurons];
            outputWeights = new double[hiddenNeurons * outputNeurons];
            for(int i = 0; i < inputNeurons * hiddenNeurons; i++)
            {
                weights[i] = r.NextDouble();
                if (r.Next() % 2 == 0)
                {
                    weights[i] = -weights[i];
                }
            }

            for(int i=0; i < hiddenNeurons * outputNeurons; i++)
            {
                outputWeights[i] = r.NextDouble();
                if (r.Next() % 2 == 0)
                {
                    outputWeights[i] = -outputWeights[i];
                }
            }
        }

        public NeuralNetwork(NeuralNetwork n)
        {
            this.weights = n.weights;
            this.outputWeights = n.outputWeights;
        }

        public double getWeigth(int inputN, int hiddenN)
        {
            return weights[hiddenN * inputNeurons + inputN];
        }

        public double getOutputWeights(int hiddenN, int outputN)
        {
            return outputWeights[hiddenNeurons * outputN + hiddenN];
        }

        public double getHiddenNeuronActivation(int index, int[] inputVector)
        {
            if(index > hiddenNeurons)
            {
                return 0;
            }
            else
            {
                double activation = 0;
                for(int i = 0; i < inputNeurons; i++)
                {
                    activation += inputVector[i] * getWeigth(i, index);
                }
                activation = 1 / (1 + Math.Exp(-activation));
                return activation;
            }
        }

        public double getOutputNeuronActivation(int index, double[] hiddenVector)
        {
            if (index > outputNeurons)
            {
                return 0;
            }
            else
            {
                double activation = 0;
                for (int i = 0; i < hiddenNeurons; i++)
                {
                    activation += hiddenVector[i] * getOutputWeights(i, index);
                }
                activation = 1 / (1 + Math.Exp(-activation));
                return activation;
            }
        }

        public double[] calculateNeuralNetwork(int[] inputVector)
        {
            double[] hiddenVector = new double[hiddenNeurons];
            for(int i = 0; i < hiddenNeurons; i++)
            {
                hiddenVector[i] = getHiddenNeuronActivation(i, inputVector);
            }
            double[] outputVector = new double[outputNeurons];
            for(int i = 0; i < outputNeurons; i++)
            {
                outputVector[i] = getOutputNeuronActivation(i, hiddenVector);
            }
            return outputVector;
        }
    }
}
