﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RobotSimulation.RobotSimulation
{
    class RobotConfiguration
    {
        double x;
        double y;
        double facing;
        const double speed = 1;
        const double wheelDist = 6;
        const double sensorDist = 8;
        const double sensorSpread = 1;

        public RobotConfiguration(double x, double y, double facing)
        {
            this.x = x;
            this.y = y;
            this.facing = facing % (Math.PI*2);
        }

        public void DecideMove(double n1, double n2)
        {
            if(Math.Abs(n1 - n2) < 0.1)
            {
                Move(3);
            }
            else
            {
                if(n1 > n2)
                {
                    Move(1);
                }
                else
                {
                    Move(2);
                }
            }
        }

        public void Move(int dir)
        {
            switch (dir)
            {
                case 1:
                    RotateLeft();
                    break;
                case 2:
                    RotateRight();
                    break;
                default:
                    GoForward();
                    break;
            }
        }

        private void GoForward()
        {
            this.x = this.x + speed * Math.Cos(this.facing);
            this.y = this.y + speed * Math.Sin(this.facing);
        }

        private void RotateRight()
        {
            double prevFac = facing;
            double newFac = (facing + speed / wheelDist) % (Math.PI * 2);
            Point center = RotatePoint(new Point(this.x, this.y), this.GetRightWheel(), prevFac - newFac);

            this.x = center.x;
            this.y = center.y;
            this.facing = newFac;
        }

        private void RotateLeft()
        {
            double prevFac = facing;
            double newFac = (facing - speed / wheelDist) % (Math.PI * 2);
            Point center = RotatePoint(new Point(this.x, this.y), this.GetRightWheel(), prevFac - newFac);

            this.x = center.x;
            this.y = center.y;
            this.facing = newFac;

        }

        public Point GetRightWheel() {
            double x = this.x + Math.Cos(this.facing + Math.PI / 2)*wheelDist;
            double y = this.y + Math.Sin(this.facing + Math.PI / 2)*wheelDist;
            Point rightWheel = new Point(x,y);
            return rightWheel;
        }

        public Point GetLeftWheel()
        {
            double x = this.x + Math.Cos(this.facing - Math.PI / 2)* wheelDist;
            double y = this.y + Math.Sin(this.facing - Math.PI / 2)* wheelDist;
            Point leftWheel = new Point(x, y);
            return leftWheel;
        }

        public Point GetSensor(int index)
        {
            double x = this.x + Math.Cos(this.facing) * sensorDist;
            double y = this.y + Math.Sin(this.facing) * sensorDist;
            double dist = 0.5 * sensorSpread;
            dist += (2-index) * sensorSpread;

            x = x + Math.Cos(this.facing - Math.PI)* dist;
            y = y + Math.Sin(this.facing - Math.PI) * dist;
            return new Point(x, y);
        }

        static Point RotatePoint(Point pointToRotate, Point centerPoint, double radians)
        {
            double cosTheta = Math.Cos(radians);
            double sinTheta = Math.Sin(radians);
            double x = cosTheta * (pointToRotate.x - centerPoint.y) -  sinTheta * (pointToRotate.x - centerPoint.y) + centerPoint.x;
            double y = sinTheta * (pointToRotate.x - centerPoint.x) + cosTheta * (pointToRotate.y - centerPoint.y) + centerPoint.y;

            return new Point(x, y);
        }
    }

}
